/*
    Project: TCTEMP
    Version: V1.18
    File   : na_sysinfo.c

    Copyright (c) 2006-2009 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "na_priv.h"

#define SystemTimeOfDayInformation 3
typedef struct
{
    LARGE_INTEGER liReserved;
    LARGE_INTEGER liReserved2;
    LARGE_INTEGER liReserved3;
    ULONG         ulReserved4;
} SYSTEM_TIME_OF_DAY_INFORMATION;

#define SystemProcessorTimes 8
typedef struct
{
    LARGE_INTEGER liReserved;
    LARGE_INTEGER liReserved2;
    LARGE_INTEGER liReserved3;
    LARGE_INTEGER liReserved4;
    LARGE_INTEGER liReserved5;
    ULONG         ulReserved6;
} SYSTEM_PROCESSOR_TIMES;

#define SystemProcessorStatistics 23
typedef struct
{
    ULONG ulReserved;
    ULONG ulReserved2;
    ULONG ulReserved3;
    ULONG ulReserved4;
    ULONG ulReserved5;
    ULONG ulReserved6;
} SYSTEM_PROCESSOR_STATISTICS;

BOOL NAAPI NaGetSystemInformation(SYSTEM_INFORMATION_CLASS sic,
  PVOID pBuffer, ULONG cbBufferSize, PULONG pcbReturnLength)
{
    NTSTATUS  ntStatus;
    ULONG     cbReturnLength;

    NaAssert(pBuffer);

    if (sic == SystemTimeOfDayInformation) {
        if (cbBufferSize > sizeof(SYSTEM_TIME_OF_DAY_INFORMATION)) {
            cbBufferSize = sizeof(SYSTEM_TIME_OF_DAY_INFORMATION);
        }
    } else if (sic == SystemProcessorTimes) {
        if (cbBufferSize > sizeof(SYSTEM_PROCESSOR_TIMES)) {
            cbBufferSize = sizeof(SYSTEM_PROCESSOR_TIMES);
        }
    }

    cbReturnLength = 0;
    ntStatus = NtQuerySystemInformation(sic, pBuffer, cbBufferSize,
      &cbReturnLength);
    if (!NT_SUCCESS(ntStatus)) {
        NaDbgPrintf(L"NaGetSystemInformation failed (0x%08x, sic=%d, len=%d)!",
            ntStatus, sic, cbReturnLength);
    }

    if (cbReturnLength == 0) {
        if (sic == SystemProcessorStatistics &&
          cbBufferSize >= sizeof(SYSTEM_PROCESSOR_STATISTICS)) {
            cbReturnLength = sizeof(SYSTEM_PROCESSOR_STATISTICS);
        }
    }

    if (pcbReturnLength) {
        *pcbReturnLength = cbReturnLength;
    }

    return TRUE;
}
